// ========================================
// Ladybird / Coccinelle
// -----------------------------------------
// Made for Persistence of vision 3.6
//==========================================  
// Copyright 2007 Gilles Tran http://www.oyonale.com
// -----------------------------------------
// This work is licensed under the Creative Commons Attribution License. 
// To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/ 
// or send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
// You are free:
// - to copy, distribute, display, and perform the work
// - to make derivative works
// - to make commercial use of the work
// Under the following conditions:
// - Attribution. You must give the original author credit.
// - For any reuse or distribution, you must make clear to others the license terms of this work.
// - Any of these conditions can be waived if you get permission from the copyright holder.
// Your fair use and other rights are in no way affected by the above. 
//========================================== 
// Render with +W800 +H600
// no antialiasing needed if the focal blur is turned on
#include "colors.inc"
#declare BugOK=1;
#declare FocalBlurOK=1;
#declare AreaOK=1;
#declare RadOK=1;
global_settings {
#if (RadOK=1)
    radiosity {
        pretrace_start 0.08           // start pretrace at this size
        pretrace_end   0.04           // end pretrace at this size
        count 300                      // higher -> higher quality (1..1600) [35]
        nearest_count 5               // higher -> higher quality (1..10) [5]
        error_bound 0.1               // higher -> smoother, less accurate [1.8]
        recursion_limit 1             // how much interreflections are calculated (1..5+) [3]
        low_error_factor .5           // reduce error_bound during last pretrace step
        gray_threshold 0.0            // increase for weakening colors (0..1) [0]
        minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
        brightness 1                  // brightness of radiosity effects (0..1) [1]
        
        adc_bailout 0.01/2
        normal on                   // take surface normals into account [off]
    }
#end
    assumed_gamma 0.7 // if the image is too dark/too light, adjust the gamma to you display_gamma setting
    max_trace_level 10
}
#declare T=transform{
    rotate -x*5
    rotate y*40
}
camera {
    location -z*20
    direction z*2.5
    up y
    right x*image_width/image_height
    look_at y*0
    #if (FocalBlurOK=1)
            aperture 0.4           // [0...N] larger is narrower depth of field (blurrier)
            blur_samples 100        // number of rays per pixel for sampling
            focal_point <0,0,0>    // point that is in focus <X,Y,Z>
            confidence 0.95           // [0...<1] when to move on while sampling (smaller is less accurate)
            variance 1/200            // [0...1] how precise to calculate (smaller is more accurate)
    #end
    transform{T}
}

// Orthographic camera used to position the legs
/*
camera {
  orthographic
  location <0,0,-100>    // position & direction of view
  look_at  <0,0,0>
  right 8*x            // horizontal size of view
  up 6*y               // vertical size of view
}
*/

#declare T_Background=texture{
    pigment{image_map{jpeg "background"}}
    finish{ambient 1 diffuse 0}
    translate -x*0.5-y*0.5
    scale <4,3,1>*30
}

plane{
    z,0
    texture{T_Background}
    no_shadow
    no_reflection
    hollow
    translate z*100
    transform{T}
}

light_source {
    -z*1000
    color rgb <255,230,200>*2/255
    #if (AreaOK=1)
        area_light 100*z,100*y, 5,5 adaptive 1 jitter orient circular
    #end
    rotate x*70
    rotate y*200
}
/*
sphere{0,2000
    texture{
        pigment{
            image_map{jpeg "background_blurred" map_type 1}
        }
        finish{ambient 1 diffuse 0 specular 0}           
    }
    no_shadow
    hollow
}
*/    
sky_sphere{
    pigment{
        image_map{jpeg "background" map_type 1}
    }
}

// -----------------------------------------
// Stem
// -----------------------------------------

#declare C_Leaf=rgb <115,139,1>/255;
#declare C_Stem=rgb <108,79,25>/255;
#declare T_Stem=texture{
    pigment{
        bozo
        turbulence 1
        color_map{
            [0  C_Leaf]
            [1  C_Leaf*0.5]
        }
    }
    normal{crackle 0.1 turbulence 0.4 lambda 3 scale <4,4,1/5>*0.3}
    finish{ambient 0 diffuse 1 specular 1 roughness 1/30}
    scale <1/4,1/4,50>
}
#declare Stem0=cylinder{-z*30,z*30,1.3 texture{T_Stem}}

#declare Thorn=cone{0,0.02, y*0.2,0}
#declare Stem=union{
    object{Stem0}
    #declare i=0;
    #declare rd=seed(1890);
    union{
        #while (i<400)
            object{Thorn
                scale <1,0.7+rand(rd)*1,1>
                rotate z*rand(rd)*20
                rotate y*360*rand(rd)
                translate y*1.3
                rotate z*360*rand(rd)
                translate z*(-20*rand(rd)+20*rand(rd))
            }
            #declare i=i+1;
        #end
        texture{pigment{rgbt <0.9,0.9,0.9,0.5>} finish{ambient 0 diffuse 0 reflection{0.2,1}}}
        interior{ior 1.4}
    }
    texture{T_Stem}
}


// -----------------------------------------
// Ladybird
// -----------------------------------------
#if (BugOK=1)
#declare F_Ladybird= finish{
    ambient 0 diffuse 1 roughness 1/100 specular 1
    reflection {0,1 metallic}
}

#declare T_Head =material{texture{
    pigment{
        image_map{jpeg "head.jpg" interpolate 2}
    }
    finish{F_Ladybird} 
}
}
#declare T_Thorax =material{texture{
    pigment{
        image_map{jpeg "thorax.jpg" interpolate 2}
    }
    finish{F_Ladybird} 
}
}
#declare T_Legs =material{texture{
    pigment{
        rgb 0.02
    }
    finish{F_Ladybird} 
}
}
#declare T_Wings = material{texture{
    pigment{
        image_map{jpeg "wings.jpg" interpolate 2}
    }
    finish{F_Ladybird} 
    }
}

#include "ladybird.inc"
// These are the original positions of meshes of the leg parts
// (see macro MoveLeg)
// DO NOT CHANGE THEM !!!
#declare PosFrontFemur=<-65.465,42.464,-100.623>;
#declare PosFrontTibia=<-145.2,39.865,-148.391>;
#declare PosFrontTarsus1=<-222.927,15.864 ,-203.554>;
#declare PosFrontTarsus2=<-243.078,8.671,-223.777>;
#declare PosFrontClaw=<-252.085,10.222,-238.298>;

#declare PosMiddleFemur=<-68.809,42.466,-46.6>;
#declare PosMiddleTibia=<-159.03,63.836 ,-53.74>;
#declare PosMiddleTarsus1=<-242.157,11.396 ,-52.838 >;
#declare PosMiddleTarsus2=<-271.578,11.216 ,-53.653>;
#declare PosMiddleClaw=<-288.353,7.656,-54.187 >;

#declare PosRearFemur=<-65.464,42.465,-3.661>;
#declare PosRearTibia=<-154.577 ,44.933,76.105>;
#declare PosRearTarsus1=<-257.757 ,8.938,151.432>;
#declare PosRearTarsus2=<-287.162,10.327,179.324>;
#declare PosRearClaw=<-303.837 ,9.108 ,196.038>;

// These are the rotation vectors for the leg parts
// You can change these to move the legs
// <0,0,0> will give the default position, with
// the ladybug resting on a flat plane
#declare RotFrontFemur=<0,0,32>;
#declare RotFrontTibia=<0,0,35>;
#declare RotFrontTarsus1=<0,0,0>;
#declare RotFrontTarsus2=<0,0,0>;
#declare RotFrontClaw=<0,0,0>;

#declare RotMiddleFemur=<0,30,32>;
#declare RotMiddleTibia=<0,0,29>;
#declare RotMiddleTarsus1=<0,0,20>;
#declare RotMiddleTarsus2=<0,0,10>;
#declare RotMiddleClaw=<0,0,0>;

#declare RotRearFemur=<0,30,33>;
#declare RotRearTibia=<0,0,0>;
#declare RotRearTarsus1=<0,0,10>;
#declare RotRearTarsus2=<0,0,10>;
#declare RotRearClaw=<0,0,0>;

#macro MoveLeg (Position,Rotation)
    translate -Position
    rotate Rotation
    translate Position
#end

// Part of the ladybug is obtained by symmetry
// To keep it simple here, the legs movements have been
// made symmetrical too (the legs are in the "Half" object), but it's possible to make the left
// and right legs independent

#declare Ladybird=union {

    object{Head  uv_mapping material{T_Head}}
    object{Body  uv_mapping material{T_Legs}}
    object{Thorax  uv_mapping material{T_Thorax}}
    object{WingLeft  uv_mapping material{T_Wings}}
    object{WingRight  uv_mapping material{T_Wings}}

    #declare Half=union{
        //head
        
        object{Eye  uv_mapping material{T_Legs}}
        object{Antenna  uv_mapping material{T_Legs}}
        object{Mandible  uv_mapping material{T_Legs}}
        
        // front leg
        union{
            object{FrontFemur  uv_mapping material{T_Legs}}
            union{
                object{FrontTibia  uv_mapping material{T_Legs}}
                union{
                    object{FrontTarsus1  uv_mapping material{T_Legs}}
                    union{
                        object{FrontTarsus2  uv_mapping material{T_Legs}}
                        object{FrontClaw  uv_mapping material{T_Legs} MoveLeg(PosFrontClaw,RotFrontClaw)}
                        MoveLeg(PosFrontTarsus2,RotFrontTarsus2)
                    }
                    MoveLeg(PosFrontTarsus1,RotFrontTarsus1)
                }
                MoveLeg(PosFrontTibia,RotFrontTibia)
            }
            MoveLeg(PosFrontFemur,RotFrontFemur)
        }

        union{
        // middle leg
            object{MiddleFemur  uv_mapping material{T_Legs}}
            union{
                object{MiddleTibia  uv_mapping material{T_Legs}}
                union{
                    object{MiddleTarsus1  uv_mapping material{T_Legs}}
                    union{
                        object{MiddleTarsus2  uv_mapping material{T_Legs}}
                        object{MiddleClaw  uv_mapping material{T_Legs} MoveLeg(PosMiddleClaw,RotMiddleClaw)}
                        MoveLeg(PosMiddleTarsus2,RotMiddleTarsus2)
                    }
                    MoveLeg(PosMiddleTarsus1,RotMiddleTarsus1)
                }
                MoveLeg(PosMiddleTibia,RotMiddleTibia)
            }
            MoveLeg(PosMiddleFemur,RotMiddleFemur)
        }

        // rear leg
        union{
            object{RearFemur  uv_mapping material{T_Legs}}
            union{
                object{RearTibia  uv_mapping material{T_Legs}}
                union{
                    object{RearTarsus1  uv_mapping material{T_Legs}}
                    union{
                        object{RearTarsus2  uv_mapping material{T_Legs}}
                        object{RearClaw  uv_mapping material{T_Legs} MoveLeg(PosRearClaw,RotRearClaw)}
                        MoveLeg(PosRearTarsus2,RotRearTarsus2)
                    }
                    MoveLeg(PosRearTarsus1,RotRearTarsus1)
                }
                MoveLeg(PosRearTibia,RotRearTibia)
            }
            MoveLeg(PosRearFemur,RotRearFemur)
        }

    }
    object{Half}
    object{Half scale <-1,1,1>}
}
#end

// -----------------------------------------
// Ladybird on stem
// -----------------------------------------
union{
    #if (BugOK=1)
        object{Ladybird scale 1/100 translate y*1 rotate z*40}
    #end
    object{Stem}
    translate -y
}

